@@ -4,6 +4,7 @@ import re
 from pathlib import Path
 from pdb import Pdb, getsourcelines
 
+import sqlparse
 from rich import box, markup
 from rich._inspect import Inspect
 from rich.console import Console
@@ -76,9 +77,13 @@ def rich_pdb_klass(base, is_celery=False, context=None, show_layouts=True):
             from prompt_toolkit.history import FileHistory
 
             if self._ipython_history_file:
-                self.shell.debugger_history = FileHistory(self._ipython_history_file)
+                history_file = FileHistory(self._ipython_history_file)
+                self.shell.debugger_history = history_file
+                # In order to fix the error for ipython 8.x
+                self.debugger_history = history_file
+
             func = super().pt_init
-            func_args = inspect.getargspec(super().pt_init).args
+            func_args = inspect.getfullargspec(super().pt_init).args
             if "pt_session_options" in func_args:
                 func(pt_session_options)
             else:
@@ -114,7 +119,6 @@ def rich_pdb_klass(base, is_celery=False, context=None, show_layouts=True):
                     ),
                     style="warning",
                     print_layout=False,
-                    dont_escape=True,
                 )
 
         do_help.__doc__ = base.do_help.__doc__
@@ -316,11 +320,14 @@ def rich_pdb_klass(base, is_celery=False, context=None, show_layouts=True):
             """sql
             Display value in sql format.
             """
-            self.do_syntax(f"{arg},'sql'")
+            val = sqlparse.format(
+                self._getval(arg), reindent=True, keyword_case="upper"
+            )
+            self._print(val)
 
         def displayhook(self, obj):
             if obj is not None:
-                self._print(obj)
+                self._print(repr(obj))
 
         def error(self, msg):
             self._print(msg, prefix="***", style="danger", print_layout=False)
@@ -353,14 +360,43 @@ def rich_pdb_klass(base, is_celery=False, context=None, show_layouts=True):
         def message(self, msg):
             self._print(msg)
 
-        def _print(
-            self, val, prefix=None, style=None, print_layout=True, dont_escape=False
-        ):
+        def precmd(self, line):
+            if line.endswith("??"):
+                line = "pinfo2 " + line[:-2]
+            elif line.endswith("?"):
+                line = "pinfo " + line[:-1]
+
+            return super().precmd(line)
+
+        def onecmd(self, line: str) -> bool:
+            """
+            Invokes 'run_magic()' if the line starts with a '%'.
+            The loop stops of this function returns True.
+            (unless an overridden 'postcmd()' behaves differently)
+            """
+            try:
+                line = line.strip()
+                if line.startswith("%"):
+                    if line.startswith("%%"):
+                        self.error(
+                            "Cell magics (multiline) are not yet supported. "
+                            "Use a single '%' instead."
+                        )
+                        return False
+                    self.run_magic(line[1:])
+                    return False
+                return super().onecmd(line)
+
+            except Exception as e:
+                self.error(f"{type(e).__qualname__} in onecmd({line!r}): {e}")
+                return False
+
+        def _print(self, val, prefix=None, style=None, print_layout=True):
             if val == "--Return--":
                 return
 
-            if isinstance(val, str) and dont_escape is False:
-                val = markup.escape(val)
+            if isinstance(val, str) and "[" in val:
+                val = markup.render(val)
 
             kwargs = {"style": str(style)} if style else {}
             args = (prefix, val) if prefix else (val,)
@@ -399,4 +435,32 @@ def rich_pdb_klass(base, is_celery=False, context=None, show_layouts=True):
             self.shell.hooks.synchronize_with_editor(filename, lineno, 0)
             # vds: <<
 
+        def run_magic(self, line) -> str:
+            """
+            Parses the line and runs the appropriate magic function.
+            Assumes that the line is without a leading '%'.
+            """
+            magic_name, arg, line = self.parseline(line)
+            if hasattr(self, f"do_{magic_name}"):
+                # We want to use do_{magic_name} methods if defined.
+                # This is indeed the case with do_pdef, do_pdoc etc,
+                # which are defined by our base class (IPython.core.debugger.Pdb).
+                result = getattr(self, f"do_{magic_name}")(arg)
+            else:
+                magic_fn = self.shell.find_line_magic(magic_name)
+                if not magic_fn:
+                    self.error(f"Line Magic %{magic_name} not found")
+                    return ""
+                if magic_name in ("time", "timeit"):
+                    result = magic_fn(
+                        arg,
+                        local_ns={**self.curframe_locals, **self.curframe.f_globals},
+                    )
+                else:
+                    result = magic_fn(arg)
+            if result:
+                result = str(result)
+                self._print(result)
+            return ""
+
     return RichPdb
